/*  NAME:
        CTEM_Demo.cpp
 
    DESCRIPTION:
        Cryogenic Transmission Electron Microscope demonstration implementation.
 
    COPYRIGHT:
        csc583VisionLib Copyright  2005, csc583VisionLib Developers.
  
        This library is free software; you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public
        License as published by the Free Software Foundation; either
        version 2 of the License, or (at your option) any later version.
 
        This library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.
 
        You should have received a copy of the GNU Lesser General Public
        License along with this library; if not, write to the Free Software
        Foundation Inc, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
    ___________________________________________________________________________
*/

#include <glut.h>
#include <fstream>
#include <iostream>
#include <stdlib.h>
#include <math.h>
#include <time.h>
//
#if	CSC583LIB_OS_WIN32
#include <QTML.h>
#include <Movies.h>
#include <FixMath.h>
#include <QuickTimeComponents.h>
#endif

#include "ImageReader.h"
#include "ImageWriter.h"
#include "RasterImage.h"
#include "RasterImage_gray.h"
#include "RasterImage_RGB.h"
#include "RasterImage_RGBa.h"
#include "VectorField_F.h"

//
// my stuff
//
#include "collier_CSC583LIB.h"
#include "collier_ImageMath.h"
#include "collier_VectorMath.h"
#include "collier_GaussianFilter.h"
#include "collier_ImageLaPlacian_Sobel.h"
#include "collier_ImageGradient_Gaussian.h"
#include "collier_GradientGaussianFilter.h"
#include "collier_ImageZeroCrossing.h"
#include "collier_ImageFFT.h"
#include "collier_HoughTransform_circle.h"

using namespace std;
using namespace csc583Base;
using namespace csc583VideoSources;
using namespace csc583ImageProcessing;
using namespace csc583MovieEditing;

//----------------------------------------------
//	Some GUI constants
//----------------------------------------------
#define	kNbRowsSubwindows	1
#define	kNbColsSubwindows	2
#define	kNbImages			kNbRowsSubwindows*kNbColsSubwindows
//
#define	kBorder				40
#define	kDefaultHeight		512
#define	kDefaultWidth		512
#define kMaxWindowWidth		1280
#define	kMaxWindowHeight	768


#ifndef	PI
#define	PI	3.14159265
#endif

//----------------------------------------------
//	Function prototypes
//----------------------------------------------
void initializeQTEnvironment(void);
void cleanupQT(void);
void loadImage(void);	
void resetImages(void);		

void drawOneCircle(void);
void fillOneDisk(void);
void myDisplayMain(void);
void myDisplay0(void);
void myDisplay1(void);

//
void myReshape(int w, int h);
void settupCamera(int w, int h);
//
void myMenuMain(int value);
void myMenu0(int value);
void myMenu1(int value);

//
void myColorSubmenu(int value);
void myMouse(int button, int state, int ix ,int iy);
void myMouse0(int button, int state, int ix ,int iy);
void myMouse1(int button, int state, int ix ,int iy);
void myKeyboard(unsigned char c, int x, int y);
int myInit(void);
void myIdle(void);
void buildDisks(float fXc, float fYc, float fR, float fThickness, float fHeight, float fFill, float fAngle, RasterImage *gMasterImage);



//----------------------------------------------
//	File-level global variables
//----------------------------------------------
//
#define kDUPE_ANALYSIS_IMAGES 1

#define kDIM 2048 // dimensioned size for FFTs

short			wBackgroundColor = 0,
				gWinWidth,
				gWinHeight,
				gImageWidth = kDefaultWidth,
				gImageHeight = kDefaultHeight,
				//
				gCurrentFieldRes = 30;
				
int				gMainWindow,
				gSubwindow[kNbRowsSubwindows*kNbColsSubwindows];
				
char 		    szSubWindowTitle[kNbRowsSubwindows*kNbColsSubwindows][100];
char			szCursorReadout[2][100];
char			szMainWindowTitle[100];
char			szStatusTitle[100];
char			*pszImageFamily = NULL;


int				nTitleX[kNbRowsSubwindows*kNbColsSubwindows] = 
						{kBorder, 2*kBorder+kDefaultWidth},
				nMainTitleX = kBorder,
				nCursorReadoutX[2] = {kBorder,2*kBorder+kDefaultWidth},
				nStatusX = 2*kBorder+kDefaultWidth;

int				nTitleY[kNbRowsSubwindows*kNbColsSubwindows] = 
						{3*kBorder+kDefaultHeight-20, 3*kBorder+kDefaultHeight-20},
				nMainTitleY = 20,
				nCursorReadoutY[2] = {50, 50},
				nStatusY = 20;
						
bool			gInitDone = false;
bool			oShowMax0 = false;
bool			oShowMax1 = false;
bool			oTransposeAnalysis = false;

// cursor locations for graphics windows 0 and 1
int				nSelectXc0, // init in resetImages()
				nSelectYc0,
				nOldSelectXc0,
				nOldSelectYc0;
int				nSelectXc1, // init in resetImages()
				nSelectYc1,
				nOldSelectXc1,
				nOldSelectYc1;
int				nRoiXo, nRoiYo;

float			fCursorValue0, fCursorValue1;

time_t			startTime, endTime;	
clock_t			startClock, endClock;

ImageStats_Type	 Image91Stats[91][91];

ImageFFT *pFFT = NULL;
		
							
// Analysis basis images
RasterImage     *pNoiseImage = NULL;
RasterImage     *pDiscImage = NULL;
RasterImage     *pReadImage = NULL;

// template images (what we compare to)
RasterImage     *pAnalysisTemplateImage = NULL;
RasterImage     *pAnalysisTemplateFFT = NULL;
RasterImage     *pMayaTemplateImage = NULL;
RasterImage     *pMayaTemplateFFT = NULL;
RasterImage     *pBloodTemplateImage = NULL;
RasterImage     *pBloodTemplateFFT = NULL;

// smaller templates for correlation processing
RasterImage		*pAnalysisCorrTempImage = NULL;
RasterImage		*pMayaCorrTempImage = NULL;
RasterImage		*pBloodCorrTempImage = NULL;

// the source images
RasterImage     *pAnalysisImage = NULL;
RasterImage     *pAnalysisFFT = NULL;
RasterImage     *pMayaImage = NULL;
RasterImage     *pMayaFFT = NULL;
RasterImage     *pBloodImage = NULL;
RasterImage     *pBloodFFT = NULL;

// the results images
RasterImage     *pAnalysisResultsImage = NULL;
RasterImage     *pAnalysisResultsFFT = NULL;
RasterImage     *pMayaResultsImage = NULL;
RasterImage     *pMayaResultsFFT = NULL;
RasterImage     *pBloodResultsImage = NULL;
RasterImage     *pBloodResultsFFT = NULL;

// currently used images
RasterImage		*pCurrentImage = NULL;
RasterImage		*pCurrentTemplateImage = NULL;
RasterImage		*pCurrentCorrTempImage = NULL;
RasterImage		*pCurrentResultsImage = NULL;

RasterImage		*pCurrentFFT = NULL;
RasterImage		*pCurrentTemplateFFT = NULL;
RasterImage		*pCurrentResultsFFT = NULL;

RasterImage		*pCurrentlyDisplayedImage = NULL;

// display raster images
RasterImage     **gMyImage = NULL;	//	an array of references to Image objects


//
//	For menu handling demo
//
#define	kNbBckgndCols		5
//
GLfloat	kBackgroundColor[kNbBckgndCols][4] =	{	{0.f, 0.f, 0.f, 1.f},	//	black
													{1.f, 0.f, 0.f, 1.f},	//	red
													{0.f, 1.f, 0.f, 1.f},	//	green
													{1.f, 1.f, 0.f, 1.f},	//	yellow
													{0.f, 1.f, 1.f, 1.f}	//	cyan
												};
GLfloat	kTitleColor[kNbBckgndCols][4] =	{	{1.f, 1.f, 1.f, 1.f},	//	white for black
											{0.f, 0.f, 0.f, 1.f},	//	black for red
											{0.f, 0.f, 0.f, 1.f},	//	black for green
											{0.f, 0.f, 0.f, 1.f},	//	black for yellow
											{0.f, 0.f, 0.f, 1.f}	//	black for cyan
										 };
char	*kBckgndColStr[] =	{
								"Black",
								"Red", 
								"Green" ,
								"Yellow",
								"Cyan"
							};	


//=============================================================================
//	Function Implementations
//=============================================================================

// test fft code
void testOOURA(void)
{
// OOURA FFT test code

#include "OOURA_alloc.h"
#include "OOURA_fft4f2d.h"

	ImageMath	ImageFilter;

	RasterImage  *pAfloat = new RasterImage_gray_F(kDIM,kDIM);
	float** fA	= ((RasterImage_gray_F*)pAfloat)->getGrayF2D();

	ImageStats_Type	 ImageStats1,ImageStats2,ImageStats3;
	ImageStats1.nHistBins = 10;
	ImageStats1.fHistMin = 0.0;
	ImageStats1.fHistMax = 1.0;

	ImageStats2.nHistBins = 10;
	ImageStats2.fHistMin = 0.0;
	ImageStats2.fHistMax = 1.0;

	ImageStats3.nHistBins = 10;
	ImageStats3.fHistMin = 0.0;
	ImageStats3.fHistMax = 1.0;

    int *ip, n1, n2, n, *ipf;
    double **a, *w, err;
    float *fW;

	n1=n2=2048;

    a = alloc_2d_double(n1, n2);
    
    n = MAX(n1, n2 / 2);
    ip = alloc_1d_int(2 + (int) sqrt(n + 0.5));
    ipf = alloc_1d_int(2 + (int) sqrt(n + 0.5));
    
    n = MAX(n1, n2) * 3 / 2;
    w = alloc_1d_double(n);
    fW = alloc_1d_float(n);
    
    ip[0] = 0;
    ipf[0] = 0;


 #if 0
    /* check of RDFT */
    putdata2d(n1, n2, a);
    startClock = clock();
    rdft2d(n1, n2, 1, a, ip, w);
    rdft2d(n1, n2, -1, a, ip, w);
    endClock = clock();
    err = errorcheck2d(n1, n2, 2.0 / n1 / n2, a);
    printf("rdft2d err= %g in %f seconds\n", err,((float)(endClock-startClock)/CLK_TCK));

    free_1d_double(w);
    free_1d_int(ip);
    free_2d_double(a);
#endif
    
    // check float version of RDFT
    //fA = alloc_2d_float(n1, n2);
    putdata2d_f(n1, n2, fA);
    
    ImageFilter.getFullStats(pAfloat,&ImageStats1);

    //startClock = clock();
    rdft2d_f(n1, n2, 1, fA, ipf, fW);
    
    ImageFilter.getFullStats(pAfloat,&ImageStats2);

    rdft2d_f(n1, n2, -1, fA, ipf, fW);
    ImageFilter.scale(pAfloat,2.0/(n1*n2));
    
    ImageFilter.getFullStats(pAfloat,&ImageStats3);

    //endClock = clock();
    err = errorcheck2d_f(n1, n2, 2.0 / n1 / n2, fA);
    //printf("rdft2d_f err= %g in %f seconds\n", err,((float)(endClock-startClock)/CLK_TCK));

	delete pAfloat;

}

//////////////////////////////////////////////////////////////////////////////////
//
// SETUP functions


void buildDiscs(float fXc, float fYc, float fRadius, float fThickness, float fHeight, float fFill, float fAngle, RasterImage *gMasterImage)
{
	ImageMath	ImageFilter;
	float fH, fR, fT;

	if (fAngle==0)
	{
		fT = 0.0f;
		// just build concentric circles with decreasing radii
		while (fT < fThickness)
		{
			ImageFilter.orCircleInto(fXc,fYc,fRadius,fFill,fAngle,gMasterImage);
			fRadius -= 0.5f;
			fT += 0.5f;
		}
	}
	else if (fAngle==90.0)
	{
		fH = 0.0f;
		fYc -= fHeight/2; // center the rectangle
		// build lines (circles at 90 deg) offset by 1 in Y.
		while (fH < fHeight)
		{
			ImageFilter.orCircleInto(fXc,fYc,fRadius,fFill,fAngle,gMasterImage);
			fYc += 1.0f;
			fH  += 1.0f;
		}
	}
	else
	{
		fH = 0.0f;
		fYc -= fHeight/2; // center the discs
		// build ovals offset by 1 in Y.
		while (fH < fHeight)
		{
			fT = 0.0f;
			fR = fRadius;
			// build concentric circles with decreasing radii
		   	while (fT < fThickness)
		   	{
		    	ImageFilter.orCircleInto(fXc,fYc,fR,fFill,fAngle,gMasterImage);
			  	fR -= 0.5;
			  	fT += 0.5;
		   	} // end thickness loop
		   	fYc += 1.0f;
		   	fH  += 1.0/sin(2*PI*fAngle/360.0f);
		} // end height loop
	}

}

//-------------------------------

void loadImage(void)			
{
	ImageMath		 ImageFilter;

	// allocate analysis basis rasters
	pNoiseImage = new RasterImage_gray_F(kDIM,kDIM);
	pDiscImage  = new RasterImage_gray_F(kDIM,kDIM);
	pReadImage  = new RasterImage_gray_F(2048,2048);

	// allocate analysis rasters
	pAnalysisTemplateImage = new RasterImage_gray_F(kDIM,kDIM);
	pAnalysisTemplateFFT   = new RasterImage_gray_F(kDIM,kDIM);
	pAnalysisImage = new RasterImage_gray_F(kDIM,kDIM);
	pAnalysisFFT   = new RasterImage_gray_F(kDIM,kDIM);
	pAnalysisResultsImage = new RasterImage_gray_F(kDIM,kDIM);
	pAnalysisResultsFFT = new RasterImage_gray_F(kDIM,kDIM);

	// allocate smaller rasters for true correlation processing
	pAnalysisCorrTempImage = new RasterImage_gray_F(99,99);
	pMayaCorrTempImage = new RasterImage_gray_F(99,99);
	pBloodCorrTempImage = new RasterImage_gray_F(99,99);

#if !kDUPE_ANALYSIS_IMAGES        // just use same rasters to save space
	// allocate source rasters
	pMayaImage     = new RasterImage_gray_F(kDIM,kDIM);
	pMayaFFT       = new RasterImage_gray_F(kDIM,kDIM);
	pMayaTemplateImage     = new RasterImage_gray_F(kDIM,kDIM);
	pMayaTemplateFFT       = new RasterImage_gray_F(kDIM,kDIM);
	pMayaResultsImage = new RasterImage_gray_F(kDIM,kDIM);
	pMayaResultsFFT = new RasterImage_gray_F(kDIM,kDIM);
	
	// allocate results rasters
	pBloodImage    = new RasterImage_gray_F(kDIM,kDIM);
	pBloodFFT      = new RasterImage_gray_F(kDIM,kDIM);
	pBloodTemplateImage    = new RasterImage_gray_F(kDIM,kDIM);
	pBloodTemplateFFT      = new RasterImage_gray_F(kDIM,kDIM);
	pBloodResultsImage = new RasterImage_gray_F(kDIM,kDIM);
	pBloodResultsFFT = new RasterImage_gray_F(kDIM,kDIM);
#endif //duplicate analysis images

	// display rendering images (only show 512x512 regions)
	gMyImage    = new RasterImage*[kNbImages];
	gMyImage[0] = new RasterImage_gray_F(512,512);
	gMyImage[1] = new RasterImage_gray_F(512,512);

	// allocate our FFT operator
	pFFT = new ImageFFT(kDIM);

	// call reset images to actually load up the images
	resetImages();
}

//-------------------------------

void resetImages(void)
{

#ifdef	MAC_VERSION
#define	mayaFilePath	    "::::Data_Samples:Gray:camera_256x256.tga"
#define	bloodFilePath	    "::::Data_Samples:Gray:camera_256x256.tga"
#else
#define mayaFilePath	    "../../../Data_Samples/Gray/Blood_013_2Kx2K_bmp.tga"
#define bloodFilePath	    "../../../Data_Samples/Gray/Blood_019_2kx2k_bmp.tga"
#endif


	ImageStats_Type	 ImageStats;
	ImageMath		 ImageFilter;

#if !kDUPE_ANALYSIS_IMAGES
	ImageReader		*myReader;
	RasterImage     *pRaster;
#endif

	ImageStats.nHistBins = 11;
	ImageStats.fHistMin = -0.1;
	ImageStats.fHistMax = 1.0;

#if 0 // test FFT code

	RasterImage  *pAfloat = new RasterImage_gray_F(kDIM,kDIM);
	RasterImage  *pBfloat = new RasterImage_gray_F(kDIM,kDIM);
	RasterImage  *pCfloat = new RasterImage_gray_F(kDIM,kDIM);
	float** fA	= ((RasterImage_gray_F*)pAfloat)->getGrayF2D();
	float** fB	= ((RasterImage_gray_F*)pBfloat)->getGrayF2D();
	float** fC	= ((RasterImage_gray_F*)pBfloat)->getGrayF2D();

	//testOOURA();


	ImageStats_Type	 ImageStats1,ImageStats2,ImageStats3, ImageStats4;
	ImageStats1.nHistBins = 11;
	ImageStats1.fHistMin = -0.1;
	ImageStats1.fHistMax = 1.0;

	ImageStats2.nHistBins = 11;
	ImageStats2.fHistMin = -0.1;
	ImageStats2.fHistMax = 1.0;

	ImageStats3.nHistBins = 11;
	ImageStats3.fHistMin = -0.1;
	ImageStats3.fHistMax = 1.0;

	ImageStats4.nHistBins = 11;
	ImageStats4.fHistMin = -0.1;
	ImageStats4.fHistMax = 1.0;

 	ImageFilter.fillUniformRandom(0.0f,1.0f,pAfloat);
    ImageFilter.getStats(pAfloat,&ImageStats4);
	
	ImageFilter.copy(pAfloat,pBfloat);
	ImageFilter.getStats(pBfloat,&ImageStats1);

	
	pFFT->setForward();
	pFFT->applyTo(pBfloat,pBfloat->getValidRect());
    ImageFilter.getStats(pBfloat,&ImageStats2);
	
	pFFT->setInverse();
	pFFT->applyInto(pBfloat,pCfloat);
    ImageFilter.getStats(pCfloat,&ImageStats3);
    
    
    ImageFilter.subtractFrom(pCfloat,pAfloat);
    ImageFilter.getStats(pAfloat,&ImageStats4);

    delete pAfloat;
    delete pBfloat;
    delete pCfloat;
    
#endif

//
// ANALYSIS RASTERS
//
	// zero out the rasters in case this is a true reset
	ImageFilter.fillInto(0.0f,pDiscImage);
	ImageFilter.fillInto(0.0f,pAnalysisTemplateImage);
	ImageFilter.fillInto(0.0f,pAnalysisCorrTempImage);

	// build background noise
	pNoiseImage->setValidRect(0,0,2048,2048); // only fill 0-2K, keep zero padding elsewhere
 	ImageFilter.fillUniformRandom(100.0f,300.0f,pNoiseImage);
    ImageFilter.getStats(pNoiseImage,&ImageStats); // for debug use
	pNoiseImage->setValidRect(pNoiseImage->getBoundRect());

	// build grid of discs from 0 to 90 degrees
	for (int nAngle = 0; nAngle<=90; nAngle++)
	//for (int nAngle = 45; nAngle<=45; nAngle++)
	{
		float fX = 185.0 + 185.0*(nAngle%10);
		float fY = 185.0 + 185.0*floor(nAngle/10);
	    //           Xc       Yc       R   Thick  Height  Fill    Angle
		buildDiscs(  fX,      fY,    43.0f, 5.0f, 20.0f, 2348.0f, (float)nAngle, pDiscImage);
		//buildDiscs(  fX,      fY,    43.0f, 5.0f, 20.0f, 2348.0f, 0.0f, pDiscImage);
	}
	
   	// set master image with template discs and noise
    ImageFilter.add(pNoiseImage, pDiscImage, pAnalysisImage);

	if (oTransposeAnalysis) ImageFilter.transpose(pAnalysisImage);


	// do FFT of analysis image
	pFFT->setForward();
	pFFT->applyInto(pAnalysisImage,pAnalysisFFT);
	
	// get a template disc
	//           Xc       Yc       R   Thick  Height  Fill    Angle
#if 1	
    buildDiscs(   0.0,     0.0,  43.0f, 5.0f, 20.0f, 1.0f, 45.0f, pAnalysisTemplateImage);
    buildDiscs(2047.0,     0.0,  43.0f, 5.0f, 20.0f, 1.0f, 45.0f, pAnalysisTemplateImage);
    buildDiscs(   0.0,  2047.0,  43.0f, 5.0f, 20.0f, 1.0f, 45.0f, pAnalysisTemplateImage);
    buildDiscs(2047.0,  2047.0,  43.0f, 5.0f, 20.0f, 1.0f, 45.0f, pAnalysisTemplateImage);
#else
    buildDiscs( 1.0*185.0, 1.0*185.0,  43.0f, 5.0f, 20.0f, 1.0f, 45.0f, pAnalysisTemplateImage);
#endif

	if (oTransposeAnalysis) ImageFilter.transpose(pAnalysisTemplateImage);

	// build a copy into smaller correlation template
	// MUST MATCH ABOVE ZERO CENTERED TEMPLATE
    buildDiscs(50.0,    50.0,    43.0f, 5.0f, 20.0f, 1.0f, 45.0f, pAnalysisCorrTempImage);

	if (oTransposeAnalysis) ImageFilter.transpose(pAnalysisCorrTempImage);


	// do FFT of analysis template image
	pFFT->setForward();
	pFFT->applyInto(pAnalysisTemplateImage,pAnalysisTemplateFFT);

#if !kDUPE_ANALYSIS_IMAGES

//
// MAYA RASTERS
//

#if 1 // read from file
	myReader = new ImageReader(mayaFilePath);
	pRaster = myReader->getFrame(true);
	delete myReader;
	
	if (pRaster ==NULL)
	{
		cout << "Error encountered while reading MAYA image file " << mayaFilePath << endl;
		std::exit(-1);
	}
	else
	{
		pRaster->copyInto(pReadImage); // rgb to gray
		ImageFilter.copyFlip(pReadImage,pMayaImage); // invert gray scales
		pMayaImage->setValidRect(0,0,kDIM,kDIM);
		delete pRaster;
	}
	
#else // just make up a set of images for now

	// zero out the rasters in case this is a true reset
	ImageFilter.fillInto(0.0f,pDiscImage);
	ImageFilter.fillInto(0.0f,pMayaTemplateImage);
	ImageFilter.fillInto(0.0f,pMayaCorrTempImage);

	// build background noise
	pNoiseImage->setValidRect(0,0,2048,2048);
 	ImageFilter.fillUniformRandom(0.0f,400.0f,pNoiseImage);
    ImageFilter.getStats(pNoiseImage,&ImageStats); // for debug use
	pNoiseImage->setValidRect(pNoiseImage->getBoundRect());

	// build grid of discs from 0 to 90 degrees
	for (int nAngle = 0; nAngle<=90; nAngle++)
	{
		float fX = 185.0 + 185.0*(nAngle%10);
		float fY = 185.0 + 185.0*floor(nAngle/10);
		//deltaR = 
	    //           Xc       Yc       R   Thick  Height  Fill    Angle
		buildDiscs(  fX,      fY,    43.0f, 5.0f, 20.0f, 2348.0f, (float)nAngle, pDiscImage);
	}
	
   	// set master image with template discs and noise
    ImageFilter.add(pNoiseImage, pDiscImage, pMayaImage);
#endif

	// do FFT of Maya image
	pFFT->setForward();
	pFFT->applyInto(pMayaImage,pMayaFFT);

	// get a template disc
	//           Xc     Yc     R   Thick  Height  Fill    Angle
    buildDiscs(   0.0,     0.0,  43.0f, 5.0f, 20.0f, 1.0f, 90.0f, pMayaTemplateImage);
    buildDiscs(2047.0,     0.0,  43.0f, 5.0f, 20.0f, 1.0f, 90.0f, pMayaTemplateImage);
    buildDiscs(   0.0,  2047.0,  43.0f, 5.0f, 20.0f, 1.0f, 90.0f, pMayaTemplateImage);
    buildDiscs(2047.0,  2047.0,  43.0f, 5.0f, 20.0f, 1.0f, 90.0f, pMayaTemplateImage);

	// build a copy into smaller correlation template
	// MUST MATCH ABOVE ZERO CENTERED TEMPLATE
    buildDiscs(50.0,    50.0,    43.0f, 5.0f, 20.0f, 1.0f, 90.0f, pMayaCorrTempImage);

	// do FFT of Maya template image
	pFFT->setForward();
	pFFT->applyInto(pMayaTemplateImage,pMayaTemplateFFT);

    
//
// BLOOD RASTERS
//

#if 1 // read from file

	myReader = new ImageReader(bloodFilePath);
	pRaster = myReader->getFrame(true);
	delete myReader;
	
	if (pRaster == NULL)
	{
		cout << "Error encountered while reading BLOOD image file " << bloodFilePath << endl;
		std::exit(-1);
	}
	else
	{
		pRaster->copyInto(pReadImage); // rgb to gray
		ImageFilter.copyFlip(pReadImage,pBloodImage); // invert gray scales
		pBloodImage->setValidRect(0,0,kDIM,kDIM);
		delete pRaster;
	}
	
#else // just make up a set of images for now

	// zero out the rasters in case this is a true reset
	ImageFilter.fillInto(0.0f,pDiscImage);
	ImageFilter.fillInto(0.0f,pBloodTemplateImage);
	ImageFilter.fillInto(0.0f,pBloodCorrTempImage);

	// build background noise
	pNoiseImage->setValidRect(0,0,2048,2048);
 	ImageFilter.fillUniformRandom(0.0f,200.0f,pNoiseImage);
    ImageFilter.getStats(pNoiseImage,&ImageStats); // for debug use
	pNoiseImage->setValidRect(pNoiseImage->getBoundRect());

	// build grid of discs from 0 to 90 degrees
	for (int nAngle = 0; nAngle<=90; nAngle++)
	{
		float fX = 185.0 + 185.0*(nAngle%10);
		float fY = 185.0 + 185.0*floor(nAngle/10);
		//deltaR = 
	    //           Xc       Yc       R   Thick  Height  Fill    Angle
		buildDiscs(  fX,      fY,    43.0f, 5.0f, 20.0f, 2348.0f, (float)nAngle, pDiscImage);
	}
	
   	// set master image with template discs and noise
    ImageFilter.add(pNoiseImage, pDiscImage, pBloodImage);
#endif

	// do FFT of Blood image
	pFFT->setForward();
	pFFT->applyInto(pBloodImage,pBloodFFT);

	// get a template disc
	//           Xc         Yc     R   Thick  Height  Fill    Angle
    buildDiscs(   0.0,     0.0,  43.0f, 5.0f, 20.0f,  1.0f, 0.0f, pBloodTemplateImage);
    buildDiscs(2047.0,     0.0,  43.0f, 5.0f, 20.0f,  1.0f, 0.0f, pBloodTemplateImage);
    buildDiscs(   0.0,  2047.0,  43.0f, 5.0f, 20.0f,  1.0f, 0.0f, pBloodTemplateImage);
    buildDiscs(2047.0,  2047.0,  43.0f, 5.0f, 20.0f,  1.0f, 0.0f, pBloodTemplateImage);

	// build a copy into smaller correlation template
	// MUST MATCH ABOVE ZERO CENTERED TEMPLATE
    buildDiscs(50.0,    50.0,    43.0f, 5.0f, 20.0f, 1.0f,   0.0f, pBloodCorrTempImage);

	// do FFT of Maya template image
	pFFT->setForward();
	pFFT->applyInto(pBloodTemplateImage,pBloodTemplateFFT);

#else
	pMayaImage     		= pAnalysisImage;
	pMayaFFT       		= pAnalysisFFT;
	pMayaTemplateImage  = pAnalysisTemplateImage;
	pMayaCorrTempImage  = pAnalysisCorrTempImage;
	pMayaTemplateFFT    = pAnalysisTemplateFFT;
	pMayaResultsImage   = pAnalysisResultsImage;
	pMayaResultsFFT     = pAnalysisResultsFFT;
	
	pBloodImage     	 = pAnalysisImage;
	pBloodFFT       	 = pAnalysisFFT;
	pBloodTemplateImage  = pAnalysisTemplateImage;
	pBloodCorrTempImage  = pAnalysisCorrTempImage;
	pBloodTemplateFFT    = pAnalysisTemplateFFT;
	pBloodResultsImage   = pAnalysisResultsImage;
	pBloodResultsFFT     = pAnalysisResultsFFT;

#endif

//
// INIT displays with analysis images
//
	nSelectXc0 = 256;
	nSelectYc0 = 256;
	nOldSelectXc0 = 256;
	nOldSelectYc0 = 256;

	nSelectXc1 = 256;
	nSelectYc1 = 256;
	nOldSelectXc1 = 256;
	nOldSelectYc1 = 256;
	
	nRoiXo = nSelectXc0-64;
	nRoiYo = nSelectYc0-64;

	pCurrentImage = pAnalysisImage;
	pCurrentTemplateImage = pAnalysisTemplateImage;
	pCurrentCorrTempImage = pAnalysisCorrTempImage;
	pCurrentFFT = pAnalysisFFT;
	pCurrentTemplateFFT = pAnalysisTemplateFFT;
	pCurrentResultsImage = pAnalysisResultsImage;
	pCurrentResultsFFT = pAnalysisResultsFFT;

	pCurrentlyDisplayedImage = pCurrentImage;

	pCurrentlyDisplayedImage->setValidRect(0,0,2048,2048);
	ImageFilter.combine(pCurrentlyDisplayedImage,4,gMyImage[0]);
	gMyImage[0]->rasterNeedsUpdate();

	// Image 1 is just created as a copy of a selected region of the master
	ImageFilter.selectRegion(pCurrentlyDisplayedImage, nSelectXc0*4, nSelectYc0*4, gMyImage[1]);
	gMyImage[1]->rasterNeedsUpdate();
	
		// init titles for sub windows, update on the fly as changes occur
	// change text in resetImages() too
	strcpy(szSubWindowTitle[0],"Analysis Image Compressed 2:1");
	strcpy(szSubWindowTitle[1],"ROI of Analysis Image");
	pszImageFamily = "Analysis";
	sprintf(szMainWindowTitle,"%s :: Image with No Processing",pszImageFamily);
	
	sprintf(szStatusTitle,"No processing done.");
	
}	


//////////////////////////////////////////////////////////////////////////////////
//
// GRAPHICS functions


void initializeQTEnvironment(void)
{
#if	CSC583LIB_OS_WIN32
	OSErr	anErr = noErr;
	anErr = InitializeQTML(kQTMLNoIdleEvents);

	if (anErr != noErr) 
	{
		cerr <<"\pQuickTime is not installed on this computer";
		ExitToShell();
	}

	
	anErr = EnterMovies();
	if(anErr != noErr)
	{
		cerr <<"\pProblems with EnterMovies, returning errors (exit)";
		ExitToShell();
	}
#endif
}

void cleanupQT(void)
{
    // terminate the QuickTime Media Layer
    ExitMovies();

#if CSC583LIB_OS_WIN32
    TerminateQTML();
#endif
}

//-------------------------------


//	draws a unit circle
void drawOneCircle(void)
{
	glBegin(GL_LINE_LOOP);
		//	in a real application, I would pre-allocate my array of vertices'
		//	coordinates and only do the drawing in this function.
		for (int i=0; i<=15; i++)
			glVertex2d(cos(2.*i*PI/16), sin(2.*i*PI/16));
	glEnd();
}

//-------------------------------

void fillOneDisk(void)
{
	glBegin(GL_POLYGON);
		//	in a real application, I would pre-allocate my array of vertices'
		//	coordinates and only do the drawing in this function.
		for (int i=0; i<=15; i++)
			glVertex2d(cos(2.*i*PI/16), sin(2.*i*PI/16));
	glEnd();
}

//-------------------------------

//	This function maps world coordinates to screen coordinates.  This is where
//	the projection mode would be defined.  Here I only want to have a 2D 
//	rectangle where to draw 2D stuff on top of my image, so I define my camera
//	as an orthographic camera, and I set the world coordinates to coincide with
//	the pixel coordinates (this is what the gluOrtho2D call does).
void settupCamera(int w, int h)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(	0.0f, (GLfloat) w, 0.0f, (GLfloat) h);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glViewport(0, 0, w, h);
}

//-------------------------------

void myReshape(int w, int h)
{
	if (gMyImage != NULL)
	{
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluOrtho2D(	0.0f, (GLfloat) w, 0.0f, (GLfloat) h);
	}
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glViewport(0, 0, w, h);
}



//////////////////////////////////////////////////////////////////////////////////
//
// DISPLAY functions

void myDisplayMain(void)
{
	ImageMath		ImageFilter;
	ImageStats_Type	ImageStats0, ImageStats1;

	glutSetWindow(gMainWindow);


	glMatrixMode(GL_MODELVIEW);
	glClear(GL_COLOR_BUFFER_BIT);
	
	// get current cursor value
	float** fRaster = ((RasterImage_gray_F*)pCurrentlyDisplayedImage)->getGrayF2D();
	fCursorValue0 = fRaster[nSelectYc0*4][nSelectXc0*4];
	fCursorValue1 = fRaster[nSelectYc1+(nRoiYo*4)][nSelectXc1+(nRoiXo*4)];

	// compute stats of images for titles
	pCurrentlyDisplayedImage->setValidRect(0,0,2048,2048);
	ImageFilter.getStats(pCurrentlyDisplayedImage,&ImageStats0); // show stats for entire image, not combined
	ImageFilter.getStats(gMyImage[1],&ImageStats1); // show stats for just the displayed ROI
	sprintf(szSubWindowTitle[0],"Mean = %.4f, min = %.4f, max = %.4f",ImageStats0.fMean, ImageStats0.fMin, ImageStats0.fMax);
	sprintf(szSubWindowTitle[1],"Mean = %.4f, min = %.4f, max = %.4f",ImageStats1.fMean, ImageStats1.fMin, ImageStats1.fMax);
	sprintf(szCursorReadout[0],"X = %d, Y = %d, value = %.4f",nSelectXc0*4,nSelectYc0*4,fCursorValue0);
	sprintf(szCursorReadout[1],"X = %d, Y = %d, value = %.4f",nSelectXc1+(nRoiXo*4),nSelectYc1+(nRoiYo*4),fCursorValue1);

	// update sub-window titles
	// do it here since they are really in main window
	glColor3fv(kTitleColor[wBackgroundColor]);
	
	for (int nWin=0; nWin<(kNbRowsSubwindows*kNbColsSubwindows); nWin++)
	{
		glRasterPos2i(nTitleX[nWin], nTitleY[nWin]);
		for (int i=0; i<strlen(szSubWindowTitle[nWin]); i++)
		{
			glutBitmapCharacter(GLUT_BITMAP_9_BY_15,(int)szSubWindowTitle[nWin][i]);
		}
	}

	// print out the main window title
	glRasterPos2i(nMainTitleX, nMainTitleY);
	for (int i=0; i<strlen(szMainWindowTitle); i++)
	{
		glutBitmapCharacter(GLUT_BITMAP_9_BY_15,(int)szMainWindowTitle[i]);
	}

	for (int j=0; j<2; j++)
	{
	   // print out the cursor readout
	   glRasterPos2i(nCursorReadoutX[j], nCursorReadoutY[j]);
	   for (int i=0; i<strlen(szCursorReadout[j]); i++)
	   {
		   glutBitmapCharacter(GLUT_BITMAP_9_BY_15,(int)szCursorReadout[j][i]);
	   }
	}
	
	// print out the overall status
	glRasterPos2i(nStatusX, nStatusY);
	for (int i=0; i<strlen(szStatusTitle); i++)
	{
		glutBitmapCharacter(GLUT_BITMAP_9_BY_15,(int)szStatusTitle[i]);
	}

	glutSwapBuffers();

	myDisplay0();
	myDisplay1();
	
}

//-------------------------------

void myDisplay0(void)
{
	ImageMath		ImageFilter;
	ImageStats_Type	ImageStats;

	glutSetWindow(gSubwindow[0]);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glClear(GL_COLOR_BUFFER_BIT);


	if (gInitDone)
	{
		gMyImage[0]->render();

		int nXo = nSelectXc0 - 64,
			nXn = nSelectXc0 + 64,
			nYo = nSelectYc0 - 64,
			nYn = nSelectYc0 + 64;
			
		if (nXo < 0) 	{nXo = 0;   nXn = 128;}
		if (nXn > 512) 	{nXn = 512; nXo = 384;}
		if (nYo < 0) 	{nYo = 0;   nYn = 128;}
		if (nYn > 512) 	{nYn = 512; nYo = 384;}

		//	draws a square marking the region being viewed in window 1
		glBegin(GL_LINE_LOOP);
			glColor3f(1.0, 0.0, 0.0);
			glVertex2i(nXo, 512-nYo);
			glVertex2i(nXo, 512-nYn);
			glVertex2i(nXn, 512-nYn);
			glVertex2i(nXn, 512-nYo);
		glEnd();
		
		if (oShowMax0)
		{
			pCurrentlyDisplayedImage->setValidRect(0,0,2048,2048);
			ImageFilter.getStats(pCurrentlyDisplayedImage,&ImageStats);

			glBegin(GL_LINE_LOOP);
				glColor3f(1.0, 0.0, 0.0);
				glVertex2i(ImageStats.nMaxX/4, 512-ImageStats.nMaxY/4-20);
				glVertex2i(ImageStats.nMaxX/4, 512-ImageStats.nMaxY/4+20);
			glEnd();
			glBegin(GL_LINE_LOOP);
				glColor3f(1.0, 0.0, 0.0);
				glVertex2i(ImageStats.nMaxX/4-20, 512-ImageStats.nMaxY/4);
				glVertex2i(ImageStats.nMaxX/4+20, 512-ImageStats.nMaxY/4);
			glEnd();
		
		}
	}

	glutSwapBuffers();

	glutSetWindow(gMainWindow);
}

//-------------------------------

void myDisplay1(void)
{
	ImageMath		ImageFilter;
	ImageStats_Type	ImageStats;

	glutSetWindow(gSubwindow[1]);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glClear(GL_COLOR_BUFFER_BIT);

	if (gInitDone)
	{
		gMyImage[1]->render();		

		if (oShowMax1)
		{
			ImageFilter.getStats(gMyImage[1],&ImageStats);

			glBegin(GL_LINE_LOOP);
				glColor3f(1.0, 0.0, 0.0);
				glVertex2i(ImageStats.nMaxX, 512-ImageStats.nMaxY-20);
				glVertex2i(ImageStats.nMaxX, 512-ImageStats.nMaxY+20);
			glEnd();
			glBegin(GL_LINE_LOOP);
				glColor3f(1.0, 0.0, 0.0);
				glVertex2i(ImageStats.nMaxX-20, 512-ImageStats.nMaxY);
				glVertex2i(ImageStats.nMaxX+20, 512-ImageStats.nMaxY);
			glEnd();
		
		}
	}

	glutSwapBuffers();

	glutSetWindow(gMainWindow);
}



//////////////////////////////////////////////////////////////////////////////////
//
// IDLE function

void myIdle(void)
{
//  nothing to do in this application
//glutPostRedisplay();
}


//////////////////////////////////////////////////////////////////////////////////
//
// KEYBOARD functions

void myKeyboard(unsigned char c, int x, int y)
{
	int nX = x;
	int nY = y;
	
	switch (c)
	{
		case 'q':
			cleanupQT( );
			std::exit(0);
			break;
			
		//	[ and ] do something to the images
		case '[':
			std::exit(0);
			break;
		case ']':
			break;
			
		//
		// use lower case r,l,u,d for left window
		//	
						
		case 'l':
			nSelectXc0--;
			cout << "LEFT  cursor in Window0 to " << nSelectXc0 << "," << nSelectYc0 << endl; 
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
						
		case 'r':
			nSelectXc0++;
			cout << "RIGHT cursor in Window0 to "  << nSelectXc0 << "," << nSelectYc0 << endl; 
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
						
		case 'u':
			nSelectYc0--;
			cout << "UP    cursor in Window0 to "  << nSelectXc0 << "," << nSelectYc0 << endl;
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
						
		case 'd':
			nSelectYc0++;
			cout << "DOWN  cursor in Window0 to "  << nSelectXc0 << "," << nSelectYc0 << endl; 
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
						
		//
		// use upper case R,L,U,D for right window
		//	
		case 'L':
			nSelectXc1--;
			cout << "LEFT  cursor in Window1 to " << nSelectXc1 << "," << nSelectYc1 << endl; 
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
						
		case 'R':
			nSelectXc1++;
			cout << "RIGHT cursor in Window1 to " << nSelectXc1 << "," << nSelectYc1 << endl; 
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
						
		case 'U':
			nSelectYc1--;
			cout << "UP    cursor in Window1 to " << nSelectXc1 << "," << nSelectYc1 << endl; 
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
						
		case 'D':
			nSelectYc1++;
			cout << "DOWN  cursor in Window1 to " << nSelectXc1 << "," << nSelectYc1 << endl; 
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;

		default:
			cout << "UNKNOWN key " << c << "in myKeyboard()" << endl; 
			break;
		
	}
}
//-------------------------------

void myKeyboard0(unsigned char c, int x, int y)
{
	int nX = x;
	int nY = y;
	
	switch (c)
	{
		case 'q':
			cleanupQT( );
			std::exit(0);
			break;
			
		//	[ and ] do something to the images
		case '[':
			std::exit(0);
			break;
		case ']':
			break;
			
		//
		// use upper OR lower case r,l,u,d for window0
		//	
						
		case 'l':
		case 'L':
			nSelectXc0--;
			cout << "LEFT  cursor in Window0 to " << nSelectXc0 << "," << nSelectYc0 << endl; 
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
						
		case 'r':
		case 'R':
			nSelectXc0++;
			cout << "RIGHT cursor in Window0 to "  << nSelectXc0 << "," << nSelectYc0 << endl; 
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
						
		case 'u':
		case 'U':
			nSelectYc0--;
			cout << "UP    cursor in Window0 to "  << nSelectXc0 << "," << nSelectYc0 << endl;
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
						
		case 'd':
		case 'D':
			nSelectYc0++;
			cout << "DOWN  cursor in Window0 to "  << nSelectXc0 << "," << nSelectYc0 << endl; 
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
						
		default:
			cout << "UNKNOWN key " << c << "in myKeyboard0()" << endl; 

			break;
		
	}
}

//-------------------------------

void myKeyboard1(unsigned char c, int x, int y)
{
	int nX = x;
	int nY = y;
	
	switch (c)
	{
		case 'q':
			cleanupQT( );
			std::exit(0);
			break;
			
		//	[ and ] do something to the images
		case '[':
			std::exit(0);
			break;
		case ']':
			break;
			
		//
		// use upper OR lower case r,l,u,d for window1
		//	
						
		case 'L':
		case 'l':
			nSelectXc1--;
			cout << "LEFT  cursor in Window1 to " << nSelectXc1 << "," << nSelectYc1 << endl; 
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
						
		case 'R':
		case 'r':
			nSelectXc1++;
			cout << "RIGHT cursor in Window1 to " << nSelectXc1 << "," << nSelectYc1 << endl; 
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
						
		case 'U':
		case 'u':
			nSelectYc1--;
			cout << "UP    cursor in Window1 to " << nSelectXc1 << "," << nSelectYc1 << endl; 
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
						
		case 'D':
		case 'd':
			nSelectYc1++;
			cout << "DOWN  cursor in Window1 to " << nSelectXc1 << "," << nSelectYc1 << endl; 
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;

		default:
			cout << "UNKNOWN key " << c << "in myKeyboard1()" << endl; 

			break;
		
	}
}

//////////////////////////////////////////////////////////////////////////////////
//
// MOUSE functions

void myMouse(int button, int state, int ix ,int iy)
{
	int nButton = button;
	int nState = state;
	int nX = ix;
	int nY = iy;
}

//-------------------------------


//	Mouse event handling function for subwindow 0.
void myMouse0(int button, int state, int ix ,int iy)
{
	ImageMath	ImageFilter;

	// if the left mouse button is clicked, set the new  region select to the X,Y
	if(state == GLUT_DOWN && button == GLUT_LEFT_BUTTON)
	{
		nSelectXc0 = ix;
		nSelectYc0 = iy;
		
		if ( (nOldSelectXc0!=nSelectXc0) || (nOldSelectYc0!=nSelectYc0) )
		{
			// clean out any junk
			gMyImage[1]->setValidRect(gMyImage[1]->getBoundRect());
			ImageFilter.fillInto(0.0f,gMyImage[1]);
			
			pCurrentlyDisplayedImage->setValidRect(0,0,2048,2048);
			ImageFilter.selectRegion(pCurrentlyDisplayedImage, nSelectXc0*4, nSelectYc0*4, gMyImage[1]);
			gMyImage[1]->rasterNeedsUpdate();

			nOldSelectXc0 = nSelectXc0;
			nOldSelectYc0 = nSelectYc0;
			nRoiXo = min(max(nSelectXc0-64,0),511-128);
	        nRoiYo = min(max(nSelectYc0-64,0),511-128);
		}
	

		glutSetWindow(gMainWindow);
		glutPostRedisplay();
	}

	// if the middle mouse button is clicked, do nothing
	if(state == GLUT_DOWN && button == GLUT_MIDDLE_BUTTON)
	{
		glutSetWindow(gMainWindow);
		glutPostRedisplay();
	}
}

//-------------------------------

void myMouse1(int button, int state, int ix ,int iy)
{
	// if the left mouse button is clicked, set the new  region select to the X,Y
	if(state == GLUT_DOWN && button == GLUT_LEFT_BUTTON)
	{
		nSelectXc1 = ix;
		nSelectYc1 = iy;
		
		glutSetWindow(gMainWindow);
		glutPostRedisplay();
	}

	// if the middle mouse button is clicked, do nothing
	if(state == GLUT_DOWN && button == GLUT_MIDDLE_BUTTON)
	{
		glutSetWindow(gMainWindow);
		glutPostRedisplay();
	}
}

//////////////////////////////////////////////////////////////////////////////////
//
// MENU functions


void myMenuMain(int value)
{
	ImageMath		ImageFilter;
	
	switch (value)
	{
		//	0 --> Exit/Quit
		case 0:
			cleanupQT( );
			std::exit(0);
			break;
			
		case 1: // reset all windows back to original images as determined in resetImages()
			resetImages();
			
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
			
		case 2: // Use Analysis images

			// make sure they are correct transpose sense
			oTransposeAnalysis = false;
			resetImages();

			// reset cursor co-ords
			nSelectXc0 = 256;
			nSelectYc0 = 256;
			nOldSelectXc0 = 256;
			nOldSelectYc0 = 256;
			nRoiXo = min(max(nSelectXc0-64,0),511-128);
	        nRoiYo = min(max(nSelectYc0-64,0),511-128);

			// set current ptrs to analysis images
			pCurrentImage = pAnalysisImage;
			pCurrentTemplateImage = pAnalysisTemplateImage;
			pCurrentCorrTempImage = pAnalysisCorrTempImage;
			pCurrentFFT = pAnalysisFFT;
			pCurrentTemplateFFT = pAnalysisTemplateFFT;
			pCurrentResultsImage = pAnalysisResultsImage;
			pCurrentResultsFFT = pAnalysisResultsFFT;
			
			pCurrentlyDisplayedImage = pAnalysisImage;

			// update window 0 and window 1
			pCurrentlyDisplayedImage->setValidRect(0,0,2048,2048);
			ImageFilter.combine(pCurrentlyDisplayedImage,4,gMyImage[0]);
			gMyImage[0]->rasterNeedsUpdate();

			// Image 1 is just created as a copy of a selected region of the master
			ImageFilter.selectRegion(pCurrentlyDisplayedImage, nSelectXc0*4, nSelectYc0*4, gMyImage[1]);
			gMyImage[1]->rasterNeedsUpdate();

			strcpy(szSubWindowTitle[0],"Analysis Image Compressed 2:1");
			strcpy(szSubWindowTitle[1],"ROI of Analysis Image");
			pszImageFamily = "Analysis";
			sprintf(szMainWindowTitle,"%s :: Image with No Processing",pszImageFamily);
			sprintf(szStatusTitle,"No processing done.");

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
		

		case 3: // Use transposed Analysis Images

			// make sure they are correct transpose sense
			oTransposeAnalysis = true;
			resetImages();

			// reset cursor co-ords
			nSelectXc0 = 256;
			nSelectYc0 = 256;
			nOldSelectXc0 = 256;
			nOldSelectYc0 = 256;
			nRoiXo = min(max(nSelectXc0-64,0),511-128);
	        nRoiYo = min(max(nSelectYc0-64,0),511-128);

			// set current ptrs to analysis images
			pCurrentImage = pAnalysisImage;
			pCurrentTemplateImage = pAnalysisTemplateImage;
			pCurrentCorrTempImage = pAnalysisCorrTempImage;
			pCurrentFFT = pAnalysisFFT;
			pCurrentTemplateFFT = pAnalysisTemplateFFT;
			pCurrentResultsImage = pAnalysisResultsImage;
			pCurrentResultsFFT = pAnalysisResultsFFT;
			
			pCurrentlyDisplayedImage = pAnalysisImage;

			// update window 0 and window 1
			pCurrentlyDisplayedImage->setValidRect(0,0,2048,2048);
			ImageFilter.combine(pCurrentlyDisplayedImage,4,gMyImage[0]);
			gMyImage[0]->rasterNeedsUpdate();

			// Image 1 is just created as a copy of a selected region of the master
			ImageFilter.selectRegion(pCurrentlyDisplayedImage, nSelectXc0*4, nSelectYc0*4, gMyImage[1]);
			gMyImage[1]->rasterNeedsUpdate();

			strcpy(szSubWindowTitle[0],"Analysis Image Compressed 2:1");
			strcpy(szSubWindowTitle[1],"ROI of Analysis Image");
			pszImageFamily = "Analysis";
			sprintf(szMainWindowTitle,"%s :: Image with No Processing",pszImageFamily);
			sprintf(szStatusTitle,"No processing done.");

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
		
		
		case 4: // Use Maya Images
    		
			// reset cursor co-ords
			nSelectXc0 = 256;
			nSelectYc0 = 256;
			nOldSelectXc0 = 256;
			nOldSelectYc0 = 256;
			nRoiXo = min(max(nSelectXc0-64,0),511-128);
	        nRoiYo = min(max(nSelectYc0-64,0),511-128);

			// set current ptrs to Maya images
			pCurrentImage = pMayaImage;
			pCurrentTemplateImage = pMayaTemplateImage;
			pCurrentCorrTempImage = pMayaCorrTempImage;
			pCurrentFFT = pMayaFFT;
			pCurrentTemplateFFT = pMayaTemplateFFT;
			pCurrentResultsImage = pMayaResultsImage;
			pCurrentResultsFFT = pMayaResultsFFT;
			
			pCurrentlyDisplayedImage = pMayaImage;

			// update window 0 and window 1
			pCurrentlyDisplayedImage->setValidRect(0,0,2048,2048);
			ImageFilter.combine(pCurrentlyDisplayedImage,4,gMyImage[0]);
			gMyImage[0]->rasterNeedsUpdate();

			// Image 1 is just created as a copy of a selected region of the master
			ImageFilter.selectRegion(pCurrentlyDisplayedImage, nSelectXc0*4, nSelectYc0*4, gMyImage[1]);
			gMyImage[1]->rasterNeedsUpdate();

			strcpy(szSubWindowTitle[0],"Maya Image Compressed 2:1");
			strcpy(szSubWindowTitle[1],"ROI of Maya Image");
			pszImageFamily = "Maya";
			sprintf(szMainWindowTitle,"%s :: Image with No Processing",pszImageFamily);
			sprintf(szStatusTitle,"No processing done.");

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;

		case 5: // Use Blood Images

			// reset cursor co-ords
			nSelectXc0 = 256;
			nSelectYc0 = 256;
			nOldSelectXc0 = 256;
			nOldSelectYc0 = 256;
			nRoiXo = min(max(nSelectXc0-64,0),511-128);
	        nRoiYo = min(max(nSelectYc0-64,0),511-128);

			// set current ptrs to blood images
			pCurrentImage = pBloodImage;
			pCurrentTemplateImage = pBloodTemplateImage;
			pCurrentCorrTempImage = pBloodCorrTempImage;
			pCurrentFFT = pBloodFFT;
			pCurrentTemplateFFT = pBloodTemplateFFT;
			pCurrentResultsImage = pBloodResultsImage;
			pCurrentResultsFFT = pBloodResultsFFT;
			
			pCurrentlyDisplayedImage = pBloodImage;

			// update window 0 and window 1
			pCurrentlyDisplayedImage->setValidRect(0,0,2048,2048);
			ImageFilter.combine(pCurrentlyDisplayedImage,4,gMyImage[0]);
			gMyImage[0]->rasterNeedsUpdate();

			// Image 1 is just created as a copy of a selected region of the master
			ImageFilter.selectRegion(pCurrentlyDisplayedImage, nSelectXc0*4, nSelectYc0*4, gMyImage[1]);
			gMyImage[1]->rasterNeedsUpdate();
    		
			strcpy(szSubWindowTitle[0],"Blood Image Compressed 2:1");
			strcpy(szSubWindowTitle[1],"ROI of Blood Image");
			pszImageFamily = "Blood";
			sprintf(szMainWindowTitle,"%s :: Image with No Processing",pszImageFamily);
			sprintf(szStatusTitle,"No processing done.");

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;


		// last real case is change background colors
		// is done in color sub menu automatically
		
		default:	// This is not happening.  This is not happening. This is not...
			cout << "Why/how did this happen in myMenuMain?" << endl;
			break;
	
	}
}

//-------------------------------

void myMenu0(int value)
{
	ImageMath		ImageFilter;

	switch (value)
	{
		//	0 --> Exit/Quit
		case 0:
			cleanupQT( );
			std::exit(0);
			break;
		
		case 1: // reset all windows back to original images
			resetImages();

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
			
		case 2: // Toggle show max location
		
			oShowMax0 = !oShowMax0;

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;

		case 3: // Show Base Image for Selected Image Family
		
			pCurrentlyDisplayedImage = pCurrentImage;

			pCurrentlyDisplayedImage->setValidRect(0,0,2048,2048);
			ImageFilter.combine(pCurrentlyDisplayedImage,4,gMyImage[0]);
			gMyImage[0]->rasterNeedsUpdate();

			// Image 1 is just created as a copy of a selected region of the master
			ImageFilter.selectRegion(pCurrentlyDisplayedImage, nSelectXc0*4, nSelectYc0*4, gMyImage[1]);
			gMyImage[1]->rasterNeedsUpdate();

			sprintf(szMainWindowTitle,"%s :: Image with No Processing",pszImageFamily);

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;

		case 4: // Show FFT of Base Image for Selected Image Family
		
			pCurrentlyDisplayedImage = pCurrentFFT;

			pCurrentlyDisplayedImage->setValidRect(0,0,2048,2048);
			ImageFilter.combine(pCurrentlyDisplayedImage,4,gMyImage[0]);
			gMyImage[0]->rasterNeedsUpdate();

			// Image 1 is just created as a copy of a selected region of the master
			ImageFilter.selectRegion(pCurrentlyDisplayedImage, nSelectXc0*4, nSelectYc0*4, gMyImage[1]);
			gMyImage[1]->rasterNeedsUpdate();

			sprintf(szMainWindowTitle,"%s :: FFT of Base Image",pszImageFamily);

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;

		case 5: // Show Template Image for Selected Image Family
		
			pCurrentlyDisplayedImage = pCurrentTemplateImage;

			pCurrentlyDisplayedImage->setValidRect(0,0,2048,2048);
			ImageFilter.combine(pCurrentlyDisplayedImage,4,gMyImage[0]);
			gMyImage[0]->rasterNeedsUpdate();

			// Image 1 is just created as a copy of a selected region of the master
			ImageFilter.selectRegion(pCurrentlyDisplayedImage, nSelectXc0*4, nSelectYc0*4, gMyImage[1]);
			gMyImage[1]->rasterNeedsUpdate();

			sprintf(szMainWindowTitle,"%s :: Template Image with No Processing",pszImageFamily);

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;

		case 6: // Show FFT of Template Image for Selected Image Family
		
			pCurrentlyDisplayedImage = pCurrentTemplateFFT;

			pCurrentlyDisplayedImage->setValidRect(0,0,2048,2048);
			ImageFilter.combine(pCurrentlyDisplayedImage,4,gMyImage[0]);
			gMyImage[0]->rasterNeedsUpdate();

			// Image 1 is just created as a copy of a selected region of the master
			ImageFilter.selectRegion(pCurrentlyDisplayedImage, nSelectXc0*4, nSelectYc0*4, gMyImage[1]);
			gMyImage[1]->rasterNeedsUpdate();

			sprintf(szMainWindowTitle,"%s :: FFT of Template Image",pszImageFamily);

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;

		case 7: // Show Results Image for Selected Image Family
		
			pCurrentlyDisplayedImage = pCurrentResultsImage;

			pCurrentlyDisplayedImage->setValidRect(0,0,2048,2048);
			ImageFilter.combine(pCurrentlyDisplayedImage,4,gMyImage[0]);
			gMyImage[0]->rasterNeedsUpdate();

			// Image 1 is just created as a copy of a selected region of the master
			ImageFilter.selectRegion(pCurrentlyDisplayedImage, nSelectXc0*4, nSelectYc0*4, gMyImage[1]);
			gMyImage[1]->rasterNeedsUpdate();

			sprintf(szMainWindowTitle,"%s :: Results Image with No Processing",pszImageFamily);

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;

		case 8: // Show FFT of Results Image for Selected Image Family
		
			pCurrentlyDisplayedImage = pCurrentResultsFFT;

			pCurrentlyDisplayedImage->setValidRect(0,0,2048,2048);
			ImageFilter.combine(pCurrentlyDisplayedImage,4,gMyImage[0]);
			gMyImage[0]->rasterNeedsUpdate();

			// Image 1 is just created as a copy of a selected region of the master
			ImageFilter.selectRegion(pCurrentlyDisplayedImage, nSelectXc0*4, nSelectYc0*4, gMyImage[1]);
			gMyImage[1]->rasterNeedsUpdate();

			sprintf(szMainWindowTitle,"%s :: FFT of Results Image",pszImageFamily);

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;

		default:	// This is not happening.  This is not happening. This is not...
			cout << "Why/how did this happen?" << endl;
			break;
	
	}
}

//-------------------------------

void myMenu1(int value)
{
	ImageMath	ImageFilter;
	FILE *fp;
	int nXo, nXn, nYo, nYn;
	int nAngle, nAngle2;
	int nX, nY;
	
	RasterImage_gray_F *pWhole;
	RasterImage_gray_F *pMain;
	RasterImage_gray_F *pTemp;
	
	char szFileName[100];

	// get the current ROI coordinates
	nXo = nSelectXc0*4 - 256;
	nXn = nSelectXc0*4 + 256;
	nYo = nSelectYc0*4 - 256;
	nYn = nSelectYc0*4 + 256;
			
	// reset ROI coordinates if near an edge
	if (nXo < 0) 	{nXo = 0;    nXn = 512;}
	if (nXn > 2048) {nXn = 2048; nXo = 2048-512;}
	if (nYo < 0) 	{nYo = 0;    nYn = 512;}
	if (nYn > 2048) {nYn = 2048; nYo = 2048-512;}


	switch (value)
	{
		
		//	0 --> Exit/Quit
		case 0:
			cleanupQT( );
			std::exit(0);
			break;
		
		case 1: // reset all windows back to original images
			resetImages();
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;

		case 2: // Toggle show max location
		
			oShowMax1 = !oShowMax1;

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;

		case 3: // Process ROI w/ CORRELATION

			// clear all the current results
			pCurrentResultsImage->setValidRect(pCurrentResultsImage->getBoundRect());
			ImageFilter.fillInto(0.0f,pCurrentResultsImage);

    		// set ValidRects for correlation
    		pCurrentImage->setValidRect(nXo,nYo,nXn-nXo,nYn-nYo);
    		
    		// do the correlation the hard way
    		startClock = clock();
    		ImageFilter.correlate(pCurrentImage,pCurrentCorrTempImage,pCurrentResultsImage);
    		endClock = clock();
    		cout << "Correlate ROI took " << ((float)((endClock-startClock)/CLK_TCK)) << " seconds" << endl;

			// reset valid rect to 2Kx2K image now that we are done with the correlation processing
			pCurrentImage->setValidRect(0,0,2048,2048);

			// do FFT on correlated results so we have something to show
			pFFT->setForward();
			pCurrentResultsImage->setValidRect(0,0,kDIM,kDIM); // FFT all
			pFFT->applyInto(pCurrentResultsImage,pCurrentResultsFFT);

			//update status title
			sprintf(szStatusTitle,"ROI processed with real correlation.");

			// display the results by default
			myMenu0(7);

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
			
		case 4: // Process ROI w/ FFT CORRELATION

			// clear out the FFT raster for zero padding
			pCurrentFFT->setValidRect(pCurrentFFT->getBoundRect());
			ImageFilter.fillInto(0.0f,pCurrentFFT);

    		// set ValidRects for FFT
    		pCurrentImage->setValidRect(nXo,nYo,nXn-nXo,nYn-nYo);
    		ImageFilter.copyTo(pCurrentImage,nXo,nYo,pCurrentFFT);
    		pCurrentImage->setValidRect(pCurrentImage->getBoundRect());
    		pCurrentFFT->setValidRect(pCurrentFFT->getBoundRect());

			// do the FFT of the ROI
			pFFT->setForward();
    		startClock = clock();
			pFFT->applyTo(pCurrentFFT);
    		endClock = clock();
    		cout << "ROI FFT took " << ((float)(endClock-startClock))/CLK_TCK << " seconds" << endl;

			// multiply new FFT or ROI with existing FFT of template
    		startClock = clock();
			ImageFilter.multiplyComplex(pCurrentFFT,pCurrentTemplateFFT,pCurrentResultsFFT);
    		endClock = clock();
    		cout << "ROI FFT multiply took " << ((float)(endClock-startClock))/CLK_TCK << " seconds" << endl;
			
			// do inverse FFT of multiply results
			pFFT->setInverse();
    		startClock = clock();
			pFFT->applyInto(pCurrentResultsFFT,pCurrentResultsImage);
    		endClock = clock();
    		cout << "ROI IFFT took " << ((float)(endClock-startClock))/CLK_TCK << " seconds" << endl;
    		
			//update status title
			sprintf(szStatusTitle,"ROI processed with FFT correlation.");

			// display the results by default
			myMenu0(7);

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
			
		case 5: // Process Combined Image w/ CORRELATION

			pWhole = new RasterImage_gray_F(512,512);
			pMain  = new RasterImage_gray_F(512,512);
			pTemp  = new RasterImage_gray_F(25,25);
			
			// combine current image and correlation template
			ImageFilter.combine(pCurrentImage,4,pWhole);
			ImageFilter.combine(pCurrentCorrTempImage,4,pTemp);

			// correlate
    		startClock = clock();
    		ImageFilter.correlate(pWhole,pTemp,pMain);
    		time(&endTime);
    		endClock = clock();
    		cout << "Correlation on combined image took " << ((float)(endClock-startClock))/CLK_TCK << " seconds" << endl;
    		
    		// uncombine pMain back into current results
			pCurrentResultsImage->setValidRect(pCurrentResultsImage->getBoundRect());
			ImageFilter.fillInto(0.0f,pCurrentResultsImage);
    		ImageFilter.uncombine(pMain,4,pCurrentResultsImage);

			//
			// write out results to CSV file
			//
			
			fp = fopen("91x1stats_combined.csv","w+"); // file in local directory
			// get the stats around each center location		
			fprintf(fp," %s, %d","max",0);
			for (nAngle2 = 0; nAngle2<=90; nAngle2++)
			{
				nX = 185 + 185*((int)(nAngle2%10));
				nY = 185 + 185*((int)floor(nAngle2/10));
    		
    			pCurrentResultsImage->setValidRect(nX-10,nY-10,20,20);
		
				ImageFilter.getStats(pCurrentResultsImage,&Image91Stats[0][nAngle2]);
				fprintf(fp,", %f",Image91Stats[0][nAngle2].fMax);
			}
			fprintf(fp,"\n");
			fclose(fp);			
				
			// reset bounding rect to entire image
			pCurrentResultsImage->setValidRect(pCurrentResultsImage->getBoundRect());
				
 			// display the results by default
			myMenu0(7);

	   		// clean up our local mess here
    		delete pWhole;
    		delete pMain;
    		delete pTemp;
    		
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
			
		case 6: // Process entire Image w/ CORRELATION

			// correlate
    		startClock = clock();
    		ImageFilter.correlate(pCurrentImage,pCurrentCorrTempImage,pCurrentResultsImage);
    		time(&endTime);
    		endClock = clock();
    		cout << "Correlation on entire image took " << ((float)(endClock-startClock))/CLK_TCK << " seconds" << endl;
    		
			//
			// write out results to CSV file
			//
			
			fp = fopen("91x1stats_entire_corr.csv","w+"); // file in local directory
			// get the stats around each center location		
			fprintf(fp," %s, %d","max",0);
			for (nAngle2 = 0; nAngle2<=90; nAngle2++)
			{
				nX = 185 + 185*((int)(nAngle2%10));
				nY = 185 + 185*((int)floor(nAngle2/10));
    		
    			pCurrentResultsImage->setValidRect(nX-10,nY-10,20,20);
		
				ImageFilter.getStats(pCurrentResultsImage,&Image91Stats[0][nAngle2]);
				fprintf(fp,", %f",Image91Stats[0][nAngle2].fMax);
			}
			fprintf(fp,"\n");
			fclose(fp);			
				
			// reset bounding rect to entire image
			pCurrentResultsImage->setValidRect(pCurrentResultsImage->getBoundRect());
				
 			// display the results by default
			myMenu0(7);
    		
			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
			
		case 7: // Process Entire Image w/ FFT CORRELATION

			// do FFT of current image in case an ROI has messed it up
    		startClock = clock();
			pFFT->setForward();
			pFFT->applyInto(pCurrentImage,pCurrentFFT);
    		endClock = clock();
    		cout << "FFT of entire current image took " << ((float)(endClock-startClock))/CLK_TCK << " seconds" << endl;

			// multiply FFT of entire image and FFT of template image
    		startClock = clock();
			ImageFilter.multiplyComplex(pCurrentFFT,pCurrentTemplateFFT,pCurrentResultsFFT);
    		endClock = clock();
    		cout << "Entire FFT multiply took " << ((float)(endClock-startClock))/CLK_TCK << " seconds" << endl;
			
			pFFT->setInverse();
    		startClock = clock();
			pFFT->applyInto(pCurrentResultsFFT,pCurrentResultsImage);
    		endClock = clock();
    		cout << "Entire IFFT took " << ((float)(endClock-startClock))/CLK_TCK << " seconds" << endl;
			
			//update status title
			sprintf(szStatusTitle,"Entire image processed with FFT correlation.");

			fp = fopen("91x1stats_entire.csv","w+");
			// get the stats around each center location		
			fprintf(fp," %s, %d","max",0);
			for (nAngle2 = 0; nAngle2<=90; nAngle2++)
			{
				nX = 185 + 185*((int)(nAngle2%10));
				nY = 185 + 185*((int)floor(nAngle2/10));
    		
    			pCurrentResultsImage->setValidRect(nX-10,nY-10,20,20);
		
				ImageFilter.getStats(pCurrentResultsImage,&Image91Stats[0][nAngle2]);
				fprintf(fp,", %f",Image91Stats[0][nAngle2].fMax);
			}
			fprintf(fp,"\n");
			fclose(fp);			
  		
			// display the results by default
			myMenu0(7);

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
			
		case 8: // Process Combined Image w/ Template Series and CORRELATION
		
			pWhole = new RasterImage_gray_F(512,512);
			pMain  = new RasterImage_gray_F(512,512);
			pTemp  = new RasterImage_gray_F(25,25);
			
			// combine currently displayed image
			ImageFilter.combine(pCurrentImage,4,pWhole);

			for (int nRadius = 43; nRadius<=43; nRadius++)
			{
			
			for (int nHeight = 20; nHeight<=20; nHeight++)
			{
			
			for (int nThick = 5; nThick<=5; nThick++)
			{
			
			sprintf(szFileName,"91x91_R%d_H%d_T%d_stats_combined_corr.csv",nRadius,nHeight,nThick);
			fp = fopen(szFileName,"w+");
			
			for (nAngle = 0; nAngle<=90; nAngle++)
			{
				cout << "Starting entire combined processing for angle " << nAngle << endl;

				// get a template disc and combine it down to size for correlation
				pCurrentTemplateImage->setValidRect(pCurrentTemplateImage->getBoundRect());
				ImageFilter.fillInto(0.0f,pCurrentTemplateImage);
				//           Xc    Yc      R               Thick         Height          Fill    Angle
	    		buildDiscs(50.0f, 50.0f, (float)nRadius, (float)nThick, (float)nHeight,  1.0f, (float)nAngle, pCurrentCorrTempImage);
				ImageFilter.combine(pCurrentCorrTempImage,4,pTemp);

				// clear out pMain for temp space
				pMain->setValidRect(pMain->getBoundRect());
				ImageFilter.fillInto(0.0f,pMain);

				// correlate
    			startClock = clock();
    			ImageFilter.correlate(pWhole,pTemp,pMain);
    			time(&endTime);
    			endClock = clock();
    			cout << "Correlation on combined image took " << ((float)(endClock-startClock))/CLK_TCK << " seconds" << endl;

	    		// uncombine pMain back into current results
				pCurrentResultsImage->setValidRect(pCurrentResultsImage->getBoundRect());
				ImageFilter.fillInto(0.0f,pCurrentResultsImage);
	    		ImageFilter.uncombine(pMain,4,pCurrentResultsImage);

				fprintf(fp," %s, %d","max",nAngle);
				// get the stats around each center location		
				for (nAngle2 = 0; nAngle2<=90; nAngle2++)
				{
					nX = 185 + 185*((int)(nAngle2%10));
					nY = 185 + 185*((int)floor(nAngle2/10));
    			
    				pCurrentResultsImage->setValidRect(nX-10,nY-10,20,20);
		
					ImageFilter.getStats(pCurrentResultsImage,&Image91Stats[nAngle][nAngle2]);
					fprintf(fp,", %f",Image91Stats[nAngle][nAngle2].fMax);
				}
				fprintf(fp,"\n");
				
				// reset bounding rect to entire image
				pCurrentResultsImage->setValidRect(pCurrentResultsImage->getBoundRect());
				
			}

			fclose(fp);
			
			
			} // end loop over thickness
			} // end loop over height
			} // end loop over radius
			
	   		// clean up our local mess here
    		delete pWhole;
    		delete pMain;
    		delete pTemp;

 			// display the results by default
			myMenu0(7);

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;

		case 9: // Process Entire Image w/ Template Series and FFT CORRELATION

			// do FFT of current image in case an ROI has messed it up
    		startClock = clock();
			pFFT->setForward();
			pFFT->applyInto(pCurrentImage,pCurrentFFT);
    		endClock = clock();
    		cout << "FFT of entire current image took " << ((float)(endClock-startClock))/CLK_TCK << " seconds" << endl;

			for (int nRadius = 43; nRadius<=43; nRadius++)
			{
			
			for (int nHeight = 20; nHeight<=20; nHeight++)
			{
			
			for (int nThick = 5; nThick<=5; nThick++)
			{
			
				sprintf(szFileName,"91x91_R%d_H%d_T%d_stats_entire_FFT.csv",nRadius,nHeight,nThick);
				fp = fopen(szFileName,"w+");
			
				for (nAngle = 0; nAngle<=90; nAngle++)
				{
					cout << "Starting entire FFT processing for angle " << nAngle << endl;

					// get a template disc
					pCurrentTemplateImage->setValidRect(pCurrentTemplateImage->getBoundRect());
					ImageFilter.fillInto(0.0f,pCurrentTemplateImage);
					//           Xc       Yc       R   Thick  Height  Fill    Angle
   					buildDiscs(   0.0,     0.0,  (float)nRadius, (float)nThick, (float)nHeight,  2048.0f, (float)nAngle, pCurrentTemplateImage);
    				buildDiscs(2047.0,     0.0,  (float)nRadius, (float)nThick, (float)nHeight,  2048.0f, (float)nAngle, pCurrentTemplateImage);
    				buildDiscs(   0.0,  2047.0,  (float)nRadius, (float)nThick, (float)nHeight,  2048.0f, (float)nAngle, pCurrentTemplateImage);
    				buildDiscs(2047.0,  2047.0,  (float)nRadius, (float)nThick, (float)nHeight,  2048.0f, (float)nAngle, pCurrentTemplateImage);
		
					// do FFT of new template
    				startClock = clock();
					pFFT->setForward();
					pFFT->applyInto(pCurrentTemplateImage,pCurrentTemplateFFT);
    				endClock = clock();
    				cout << "FFT of template image took " << ((float)(endClock-startClock))/CLK_TCK << " seconds" << endl;

					// multiply FFT of entire image and FFT of template image
    				startClock = clock();
					ImageFilter.multiplyComplex(pCurrentFFT,pCurrentTemplateFFT,pCurrentResultsFFT);
    				endClock = clock();
    				cout << "Entire FFT multiply took " << ((float)(endClock-startClock))/CLK_TCK << " seconds" << endl;
			
					// do inverse FFT to get correlation
					pFFT->setInverse();
    				startClock = clock();
					pFFT->applyInto(pCurrentResultsFFT,pCurrentResultsImage);
    				endClock = clock();
    				cout << "Entire IFFT took " << ((float)(endClock-startClock))/CLK_TCK << " seconds" << endl;

					fprintf(fp," %s, %d","max",nAngle);
					// get the stats around each center location		
					for (nAngle2 = 0; nAngle2<=90; nAngle2++)
					{
						nX = 185 + 185*((int)(nAngle2%10));
						nY = 185 + 185*((int)floor(nAngle2/10));
    			
    					pCurrentResultsImage->setValidRect(nX-20,nY-20,40,40);
		
						ImageFilter.getStats(pCurrentResultsImage,&Image91Stats[nAngle][nAngle2]);
						fprintf(fp,", %f",Image91Stats[nAngle][nAngle2].fMax);
					}
					fprintf(fp,"\n");
				
					// reset bounding rect to entire image
					pCurrentResultsImage->setValidRect(pCurrentResultsImage->getBoundRect());
				}

			fclose(fp);
			
			
			} // end loop over thickness
			} // end loop over height
			} // end loop over radius
			
 			// display the results by default
			myMenu0(7);

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;

		case 10: // Process ROI-centered 1Kx1K of Image w/ FFT CORRELATION

			// get the current ROI coordinates
			nXo = 185*(1+(nSelectXc0*4)/185) - 512;
			nYo = 185*(1+(nSelectYc0*4)/185) - 512;
			nXn = nXo + 1023;
			nYn = nYo + 1023;
			
			// reset ROI coordinates if near an edge
			if (nXo < 0) 	{nXo = 0;    nXn = 1023;}
			if (nXn > 2048) {nXn = 2047; nXo = 2047-1024;}
			if (nYo < 0) 	{nYo = 0;    nYn = 1023;}
			if (nYn > 2048) {nYn = 2047; nYo = 2047-1024;}

			// mask off all but 1Kx1K region of the current image
			// YES, this will mess it up but the user can reset it if desired
			// Copy this to the origin

			ImageFilter.copy(pCurrentImage,pNoiseImage); // temp copy
			ImageFilter.fillInto(0.0f,pCurrentImage);
			pNoiseImage->setValidRect(nXo,nYo,1024,1024); 
			ImageFilter.copyTo(pNoiseImage, 0, 0, pCurrentImage);
			pCurrentImage->setValidRect(pCurrentImage->getBoundRect()); 

			// do FFT of current image 
    		startClock = clock();
			pFFT->setForward();
			pFFT->applyInto(pCurrentImage,pCurrentFFT);
    		endClock = clock();
    		cout << "FFT of 1Kx1K image took " << ((float)(endClock-startClock))/CLK_TCK << " seconds" << endl;

			//Build template on 185 boundary
			pCurrentTemplateImage->setValidRect(pCurrentTemplateImage->getBoundRect()); 
			ImageFilter.fillInto(0.0f,pCurrentTemplateImage);
    		buildDiscs(185.0,    185.0,    43.0f, 5.0f, 20.0f, 1.0f, 0.0f, pCurrentTemplateImage);


			// do FFT of correlation template image 
    		startClock = clock();
			pFFT->setForward();
			pFFT->applyInto(pCurrentTemplateImage,pCurrentTemplateFFT);
    		endClock = clock();
    		cout << "FFT of 1Kx1K template image took " << ((float)(endClock-startClock))/CLK_TCK << " seconds" << endl;

			// multiply FFT of entire image and FFT of template image
    		startClock = clock();
			ImageFilter.multiplyComplex(pCurrentFFT,pCurrentTemplateFFT,pCurrentResultsFFT);
    		endClock = clock();
    		cout << "Entire FFT multiply took " << ((float)(endClock-startClock))/CLK_TCK << " seconds" << endl;
			
			pFFT->setInverse();
    		startClock = clock();
			pFFT->applyInto(pCurrentResultsFFT,pCurrentResultsImage);
    		endClock = clock();
    		cout << "Entire IFFT took " << ((float)(endClock-startClock))/CLK_TCK << " seconds" << endl;
			
			//update status title
			sprintf(szStatusTitle,"1Kx1K image processed with FFT correlation.");

#if 0
			fp = fopen("91x1stats_entire.csv","w+");
			// get the stats around each center location		
			fprintf(fp," %s, %d","max",0);
			for (nAngle2 = 0; nAngle2<=90; nAngle2++)
			{
				nX = 185 + 185*((int)(nAngle2%10));
				nY = 185 + 185*((int)floor(nAngle2/10));
    		
    			pCurrentResultsImage->setValidRect(nX-10,nY-10,20,20);
		
				ImageFilter.getStats(pCurrentResultsImage,&Image91Stats[0][nAngle2]);
				fprintf(fp,", %f",Image91Stats[0][nAngle2].fMax);
			}
			fprintf(fp,"\n");
			fclose(fp);			
#endif  		
			// display the results by default
			myMenu0(7);

			glutSetWindow(gMainWindow);
			glutPostRedisplay();
			break;
			
		default:	// This is not happening.  This is not happening. This is not...
			cout << "Why is this happening in myMenu 1?" << endl;
			break;
	
	}
}

//-------------------------------


void myColorSubmenu(int colorIndex)
{
	GLfloat		*backColor;
	
	if ((colorIndex >= 0) && (colorIndex < kNbBckgndCols))
	{
		glutSetWindow(gMainWindow);

		backColor = kBackgroundColor[colorIndex];
		wBackgroundColor = colorIndex; 
		glClearColor(backColor[0], backColor[1], backColor[2], backColor[3]);

		myDisplay0();
		glutSetWindow(gMainWindow);
	    glutPostRedisplay();
	}
}


//////////////////////////////////////////////////////////////////////////////////
//
// INIT function

int myInit(void)
{
	// Create Menus
	int menuMain, menu[kNbRowsSubwindows*kNbColsSubwindows], colorSubmenu;
	
	//	Set a general menu for the main window
	glutSetWindow(gMainWindow);
	glClearColor(0.0, 0.0, 0.0, 1.0);

	//	Submenu for background color selection choices
	colorSubmenu = glutCreateMenu(myColorSubmenu);
	for (int i=0; i<kNbBckgndCols; i++)
		glutAddMenuEntry(kBckgndColStr[i], i);


	// Main menu that the submenus are connected to
	glutSetWindow(gMainWindow);
	menuMain = glutCreateMenu(myMenuMain);
	glutAddMenuEntry("Quit", 0); 
	glutAddMenuEntry("Reset Images", 1); 
	glutAddMenuEntry("Process Analysis Image", 2); 
	glutAddMenuEntry("Process Transposed Analysis Image", 3); 
	glutAddMenuEntry("Process Maya Image", 4); 
	glutAddMenuEntry("Process Blood Image", 5); 
	glutAddSubMenu("Color", colorSubmenu); 
	glutAttachMenu(GLUT_RIGHT_BUTTON);
	

	glutSetWindow(gSubwindow[0]);
	menu[0] = glutCreateMenu(myMenu0);
	glutAddMenuEntry("Quit", 0);
	glutAddMenuEntry("Reset Images", 1); 
	glutAddMenuEntry("Toggle show Max Point", 2); 
	glutAddMenuEntry("Show Base Image", 3); 
	glutAddMenuEntry("Show FFT of Base Image", 4); 
	glutAddMenuEntry("Show Template Image", 5); 
	glutAddMenuEntry("Show FFT of Template Image", 6); 
	glutAddMenuEntry("Show Results Image", 7); 
	glutAddMenuEntry("Show FFT of Results Image", 8); 
	glutAttachMenu(GLUT_RIGHT_BUTTON);
	
	
	glutSetWindow(gSubwindow[1]);
	menu[1] = glutCreateMenu(myMenu1);
	glutAddMenuEntry("Quit", 0);
	glutAddMenuEntry("Reset Images", 1); 
	glutAddMenuEntry("Toggle show Max Point", 2); 
	glutAddMenuEntry("Process ROI w/ CORRELATION", 3);
	glutAddMenuEntry("Process ROI w/ FFT CORRELATION", 4);
	glutAddMenuEntry("Process Combined Image w/ CORRELATION", 5);
	glutAddMenuEntry("Process Entire Image w/ CORRELATION", 6);
	glutAddMenuEntry("Process Entire Image w/ FFT CORRELATION", 7);
	glutAddMenuEntry("Process Combined Image w/ Template Series and CORRELATION", 8);
	glutAddMenuEntry("Process Entire Image w/ Template Series and FFT CORRELATION", 9);
	glutAddMenuEntry("Process ROI-centered 1Kx1K of Image w/ FFT CORRELATION", 10);
	glutAttachMenu(GLUT_RIGHT_BUTTON);

 
	glutSetWindow(gMainWindow);

	gInitDone = true;
	
	return false;

}

//////////////////////////////////////////////////////////////////////////////////
//
// MAIN function

int main(int argc, char** argv)
{

	int		winID;
	GLfloat	*backColor;
	void	(*displayFuncTable[3])() = {myDisplayMain, myDisplay0, myDisplay1};
	void	(*mouseFuncTable[3])(int,int,int,int) = {myMouse, myMouse0, myMouse1};
	void	(*keyboardFuncTable[3])(unsigned char, int, int) = {myKeyboard, myKeyboard0, myKeyboard1};
	
	glutInit(&argc, argv);

	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
	
	initializeQTEnvironment();

	//	I load my images
	loadImage();

	short	imgHeight = gMyImage[0]->getHeight(),
			imgWidth = gMyImage[0]->getWidth();

	//	I initialize my window's dimensions
	gWinHeight = imgHeight + 3 * kBorder;
	gWinWidth = 2 * imgWidth + 3 * kBorder;
	
	glutInitWindowSize(gWinWidth,gWinHeight);
	gMainWindow = glutCreateWindow("CTEM Demo");

	glutKeyboardFunc(myKeyboard);
	glutDisplayFunc(myDisplayMain);
	glutReshapeFunc(myReshape);
	glutIdleFunc(myIdle);
	glutMouseFunc(myMouse);

	glClearColor(0.0, 0.0, 0.0, 1.0);
	
	winID = 0;
	glutSetWindow(gMainWindow);
	for (int i=0; i<kNbRowsSubwindows; i++)
	{ 
		for (int j=0; j<kNbColsSubwindows; j++)
		{
			gSubwindow[winID] = glutCreateSubWindow(gMainWindow,
										j*(imgWidth + kBorder) + kBorder, 
										i*(imgHeight + kBorder) + kBorder,
										imgWidth, imgHeight);
														
			//	I don't need to do this.  I just set a different "clear" color for
			//	each subwindow.
			backColor = kBackgroundColor[0]; // always use black
			glClearColor(backColor[0], backColor[1], backColor[2], backColor[3]);
	
			//	This *has* to be defined:  a "display" callback function for each subwindow
			glutDisplayFunc(displayFuncTable[winID+1]);
			
			// assign a mouse handler for each window
			glutMouseFunc(mouseFuncTable[winID+1]);

			// assign a mouse handler for each window
			glutKeyboardFunc(keyboardFuncTable[winID+1]);

			//	This calls maps "world" coordinates to "screen" coordinates. 
			//	In this particular case I make them coincide.
			settupCamera(imgWidth, imgHeight);
			//
			glutSetWindow(gMainWindow);
			
			// increment winID
			winID++;
			
		}
	} 

	myInit();
	glutSetWindow(gMainWindow);
	glutPostRedisplay();
    
	glutMainLoop();
	
	cleanupQT( );
	
	return 0;
}