#ifndef IMAGE_GRADIENT_GAUSSIAN_H
#define IMAGE_GRADIENT_GAUSSIAN_H

#include "ImageOperator.h"
#include "VectorOperator.h"

using namespace csc583Base;

namespace csc583ImageProcessing
{

    class CSC583LIB_EXPORT ImageGradient : public VectorOperator
    {
        public:

            ImageGradient();

            /**
             *  Destructor
             */          
            ImageGradient(void);

            /** Applies this operator to an image, returns the result as a VectorField.
             *
             *  Since the operator is a "float" operator, then the VectorField
             *  returned should will be of the "float" type matching the input
             *  image's base raster.
             *
             *  @param  imgIn       the RasterImage to which the operator is applied
             *  @return             the VectorField obtained by applying the operator to
             *                      imgIn
             */ 
            VectorField *applyTo(RasterImage *imgIn);

            /** Applies this operator to an image  within the rectangle specified and
             *  returns the result as a VectorField.
             *
             *  The RasterImage returned should be of the "float" form of the input 
             *  image.  For example, if imgIn was RasterImage_Gray, then the image
             *  returned should be RasterImage_Gray_F.
             *
             *  @param  theRect     the rectangle within which to apply the filter
             *  @param  imgIn       the RasterImage to which the operator is applied
             *  @return             the VectorField obtained by applying the operator to
             *                      imgIn
             */ 
            VectorField *applyTo(RasterImage *imgIn, ImageRect *theRect);

            /** Applies this operator to a RasterImage and writes the result into a
             *  VectorField received as second parameter.
             *
             *  @param  imgIn       the RasterImage to which the operator is applied
             *  @param  vectOut     the VectorField into which the output is written
             */ 
            void applyInto(RasterImage *imgIn, VectorField *vectOut);

            /** Applies this operator to a RasterImage within the rectangle specified
             *  and writes the result into the VectorField received as parameter.
             *
             *  The RasterImage returned should be of the "float" form of the input 
             *  image.  For example, if imgIn was RasterImage_Gray, then the image
             *  returned should be RasterImage_Gray_F.
             *
             *  @param  imgIn       the RasterImage to which the operator is applied
             *  @param  theRect     the rectangle within which to apply the filter
             *  @param  vectOut     the VectorField into which the output is written
             */ 
            void applyInto(RasterImage *imgIn, ImageRect *theRect, VectorField *vectOut);

        protected:
        
            // anything you want

    };
}
#endif  //  IMAGE_GRADIENT_GAUSSIAN_H