#ifndef IMAGE_GRADIENT_GAUSSIAN_FILTER_H
#define IMAGE_GRADIENT_GAUSSIAN_FILTER_H

#include "VectorOperator.h"
#include "ImageOperator.h"

using namespace csc583Base;

namespace csc583ImageProcessing
{

    class CSC583LIB_EXPORT ImageGradient_Gaussian  : public ImageOperator
    {
        public:

            /** Defines a Gaussian Filter with a specified scale
             *  The relations between scale, radius and width of the filter are
             *      width = (int)(5*scale)  and  width = 2*radius+1
             *
             *  @param  theScale    scale parameter sigma of the 2D Gaussian filter
             */
            ImageGradient_Gaussian(double theScale);

            /** Defines a Gaussian Filter with a specified "radius".
             *  The scale of the filter is the square root of the variance of the
             *  associated distrinution.  It it also related to the width of the 
             *  filter by the relation
             *      width = ceil(5*scale).<p>
             *  The actual width of the filter = 2*radius + 1<p>
             *  it is set to the smallest odd number greater or equal to the parameter
             *  received.
             *  @param  theWidth   desired width of the Gaussian filter
             */
            ImageGradient_Gaussian (short theWidth);
            
            /** Destructor
             */
            ~ImageGradient_Gaussian (void);

            /** Applies this operator to an image, returns the result as a RasterImage.
             *
             *  The RasterImage returned is of the "float" form of the input image.
             *  For example, if imgIn was RasterImage_Gray, then the image
             *  returned should be RasterImage_Gray_F.
             *
             *  @param  imgIn       the RasterImage to which the operator is applied
             *  @return             the RasterImage obtained by applying the operator to
             *                      imgIn
             */
             
            //**//**//**//**//**//**//**//**//**//**//**//**//**//**//**
			//
			// !!!!! CAVEAT :: USER BEWARE !!!!!
			//
			// Returning a pointer from a function other than "new"
			// 	is never considered a good thing because of the possible
			//	risk of easily generating memory leaks. Therefore,
			//
			// THIS FUNCTION RETURNS A POINTER TO A PROTECTED VARIABLE
			//
			// Each call to this function will delete any prior results
			//	and reinit the internal variable.
			//
			//**//**//**//**//**//**//**//**//**//**//**//**//**//**//**
 
            VectorField *applyTo(RasterImage *imgIn);

            /** Applies this operator to an image  within the rectangle specified and
             *  returns the result as a RasterImage.
             *
             *  The RasterImage returned is of the "float" form of the input image.
             *  For example, if imgIn was RasterImage_Gray, then the image
             *  returned should be RasterImage_Gray_F.
             *
             *  @param  theRect     the rectangle within which to apply the filter
             *  @param  imgIn       the RasterImage to which the operator is applied
             *  @return             the RasterImage obtained by applying the operator to
             *                      imgIn
             */
             
            //**//**//**//**//**//**//**//**//**//**//**//**//**//**//**
			//
			// !!!!! CAVEAT :: USER BEWARE !!!!!
			//
			// Returning a pointer from a function other than "new"
			// 	is never considered a good thing because of the possible
			//	risk of easily generating memory leaks. Therefore,
			//
			// THIS FUNCTION RETURNS A POINTER TO A PROTECTED VARIABLE
			//
			// Each call to this function will delete any prior results
			//	and reinit the internal variable.
			//
			//**//**//**//**//**//**//**//**//**//**//**//**//**//**//**
              
            VectorField *applyTo(RasterImage *imgIn, ImageRect *theRect);

            /** Applies this operator to a RasterImage and writes the result into a second
             *  RasterImage received as parameter.
             *
             *  @param  imgIn       the RasterImage to which the operator is applied
             *  @param  imgOut      the RasterImage into which the output is written
             */ 
            void applyInto(RasterImage *imgIn, VectorField *vectOut);
            
            /** Applies this operator to a RasterImage within the rectangle specified
             *  and writes the result into the RasterImage received as parameter.
             *
             *  @param  imgIn       the RasterImage to which the operator is applied
             *  @param  theRect     the rectangle within which to apply the filter
             *  @param  imgOut      the RasterImage into which the output is written
             */ 
            void applyInto(RasterImage *imgIn, ImageRect *theRect, VectorField *vectOut);

        protected:

        	double	dSigma_;
        	short	wWidth_;
        	short	wRadius_;
        	float	fNormFactor_;
        	RasterImage_gray_F *DxRasterF_;
        	RasterImage_gray_F *DyRasterF_;
           	RasterImage_gray_F *FilterRasterF_;
	       	RasterImage_gray_F *GaussianRasterF_;
        	RasterImage_gray_F *X_RasterF_;
        	RasterImage_gray_F *Y_RasterF_;
        	
        	VectorField_F *OutVectorFieldF_;

    };
}
#endif  //  IMAGE_GRADIENT_GAUSSIAN_FILTER_H