#ifndef IMAGE_HOUGH_TRANSFORM_H
#define IMAGE_HOUGH_TRANSFORM_H

#include "ImageOperator.h"

using namespace csc583Base;

namespace csc583ImageProcessing
{

    class CSC583LIB_EXPORT ImageHoughTransform : public ImageOperator
   {
        public:

            /** Defines a generic Hough Transform class
             *
             *  @param  m    dimension of the accumulator array
             *  @param  n[]  resolution of the accumulator along each axis
             *  @param  nMin start dimension of accumulators
             *  @param  nMax end dimension of accumulators
             *
             * Let's say as well that you only care about radii in the range [3, 20], 
             * which corresponds to 35 values of r.  Then if you are working with 
             * 256x256 images (513 x 513 vlaues of cx and cy, and n[] would be {513, 513, 35},
             * nMin would be 3 and nMax is 20. Since there are 35 steps, the delta radius
             * is 1/2.  
             */
            ImageHoughTransform(int m, int n[], int nMin, int nMax);

            ImageHoughTransform(int nNumSearchs);
            
            /** Destructor
             */
            virtual ~ImageHoughTransform(void);

// stubs for virtuals from ImageOperator
RasterImage *applyTo(RasterImage *imgIn, ImageRect *theRect);
RasterImage *applyTo(RasterImage *imgIn);
void applyInto(RasterImage *imgIn, RasterImage *imgOut);
void applyInto(RasterImage *imgIn, ImageRect *theRect, RasterImage *imgOut);

            /** Applies this operator to a RasterImage which is NOT assumed to be binary.
             *
             *  @param  imgIn       the RasterImage to which the operator is applied
             */ 
            virtual void *applyTo(RasterImage_gray *imgIn) = 0;
            
            /** Applies this operator to a RasterImage within the rectangle specified
             *  and writes the result into the RasterImage received as parameter.
             *
             * This function takes in a gray-level image and a boolean that 
			 * indicates whether that image is in fact a binary image (the 
			 * output of a zero crossing or thresholding operation). It returns 
			 * its results cast as a pointer to void.
			 *
             *  @param  imgIn       the RasterImage to which the operator is applied
             *  @param  isBinary    
             */ 
            virtual void *applyTo(RasterImage_gray *imgIn, bool isBinary) = 0;

        protected:

        	bool oSearchFixedRange_;
        	int nNumSizes_, nMin_, nMax_;
        	float fDeltaDim_;
        	int *pnAccumSizes_;
        	RasterImage_gray_F **pAccumRastersGF_;
        
    };
}
#endif  //  IMAGE_HOUGH_TRANSFORM_H
